@@ -7,7 +7,11 @@ from __future__ import unicode_literals, print_function
 import io
 import re
 import logging
-from collections import OrderedDict, MutableMapping
+from collections import MutableMapping
+try:
+    from collections import OrderedDict
+except ImportError as e:
+    from ordereddict import OrderedDict
 
 
 __version__ = '0.2.9'
